using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Microsoft.Win32;

public interface IEveryDataSample
{
    string Value {set;get;}
}

namespace SampleAxControlCSharpDotNet
{
    [ProgId("MyEveryDataPointCSharpDisplays.MyCSharpDisplay")]
    [ClassInterface(ClassInterfaceType.None)]
    public partial class EveryDataPointCSharp : UserControl, IEveryDataSample, IadsAutomationLib.IAcceptIadsParameter, IadsAutomationLib.ISynchronizeOutput 
    {
        [ComRegisterFunctionAttribute]
        public static void RegisterFunction(Type key)
        {
            Registry.ClassesRoot.CreateSubKey( @"CLSID\{" + key.GUID.ToString().ToUpper() + @"}\Programmable");
            Registry.ClassesRoot.CreateSubKey( @"CLSID\{" + key.GUID.ToString().ToUpper() + @"}\Control");

            // Including a bitmap for the Iads display builder. It's a little painful because the resource must be Win32 compatable
            // 1) Create file resfile.rc with Notepad.exe in project directory
            // 2) Enter one line in the file->  102 BITMAP "bitmap1.bmp"
            // 3) Save and exit Notepad
            // 4) Create a bitmap1.bmp file in the same directory, 48x48 pixels using paint.exe or some other image editor (or dev studio)
            // 5) Select Project in solution explorer
            // 6) Project->Properties->Build Events
            // 7) Add to pre-build command line->  "C:\Program Files\Microsoft Visual Studio 8\SDK\v2.0\Bin\rc.exe" /r "$(ProjectDir)resfile.rc"
            // 8) Build project and check for errors. If no errors then resfile.res should be created in the project directory
            // 9) Add both resfile.rc and resfile.res as well as bitmap1.bmp to the project (right click on project in solution explorer->add->existing item)
            // 10) Project->Properties->Application->Resources
            // 11) Select "Resource File" radio button and browse for resfile.res that was just created
            // 12) Build and check for errors... then add the following lines and the last two lines below in the unregister regarding DefaultIcon and ToolboxBitmap32
            // 13) Build and add your control to the Iads display builder (see tutorial for more info). Icon should be present on tool
            string path = System.Reflection.Assembly.GetAssembly(typeof(EveryDataPointCSharp)).Location;
            Microsoft.Win32.RegistryKey subkey;
            subkey = Microsoft.Win32.Registry.ClassesRoot.CreateSubKey(@"CLSID\{" + key.GUID.ToString().ToUpper() + @"}\DefaultIcon");
            subkey.SetValue("", path + ",102");
            subkey = Microsoft.Win32.Registry.ClassesRoot.CreateSubKey(@"CLSID\{" + key.GUID.ToString().ToUpper() + @"}\ToolBoxBitmap32");
            subkey.SetValue("", path + ",102");
        }
        [ComUnregisterFunctionAttribute]
        public static void UnregisterFunction(Type key)
        {
            Registry.ClassesRoot.DeleteSubKey( @"CLSID\{" + key.GUID.ToString().ToUpper() + @"}\Programmable");
            Registry.ClassesRoot.DeleteSubKey( @"CLSID\{" + key.GUID.ToString().ToUpper() + @"}\Control");
            Registry.ClassesRoot.DeleteSubKey(@"CLSID\{" + key.GUID.ToString().ToUpper() + @"}\ToolBoxBitmap32");
            Registry.ClassesRoot.DeleteSubKey(@"CLSID\{" + key.GUID.ToString().ToUpper() + @"}\DefaultIcon");
        }

        IadsAutomationLib.Parameter mParameter;
        bool mThisDisplayIsInRealTime;
        //long mCounter;
        System.IO.StreamWriter mTr;

        public EveryDataPointCSharp()
        {
            InitializeComponent();
            mParameter = null;
            mThisDisplayIsInRealTime = true; // Default to true. Container will set when user scrolls back
            //mCounter = 0;
            mTr = new System.IO.StreamWriter("C:\\DataOut.txt");
        }

        public string Value
        {
            get
            {
                return label1.Text;
            }
            set
            {
                label1.Text = value;
            }
        }

        #region IAcceptIadsParameter Members

        public void Add(IadsAutomationLib.Parameter newVal, string AttachPointName, int AttachPointDepthIndex)
        {
            // Since 
            mParameter = newVal;
        }

        public int AttachPointCount
        {
            // Attach point is basically a "parameter slot", much like a property from the properties sheet
            // For this display, we only have one attachment (i.e. one parameter)
            get
            {
                return 1;
            }
        }

        public string AttachPointName(int index)
        {
            // In this function, we need to return the name of the "Nth" attach point
            if (index == 0)
                return "AttachPoint1";
            else
                return "";
        }

        public int Count
        {
            // In this function, we need to return the number of currently attached parameters
			      get
			      {
                if (mParameter != null)
                    return 1;
                else
                    return 0;
      			}
        }

        public System.Collections.IEnumerator GetEnumerator()
        {
            // Don't need to implement this function
            throw new Exception("The method or operation is not implemented.");
        }

        public IadsAutomationLib.Parameter Item(object AttachPointNameOrIndex, int AttachPointDepthIndex)
        {
            // The AttachPointNameOrIndexOrObject can be either a string (your attach point name) or an index (0..Count-1)
            // Determine the attach point and return the parameter
            string ap = Convert.ToString(AttachPointNameOrIndex);
            if (ap == "AttachPoint1" || ap == "0")
                return mParameter;
            else
                return null;
        }

        public void Remove(object AttachPointNameOrIndexOrObject, int AttachPointDepthIndex)
        {
            // The AttachPointNameOrIndexOrObject can be either a string (your attach point name), an index (0..Count-1), or the actual parameter to remove
            // Determine the attach point and remove the parameter
            if (AttachPointNameOrIndexOrObject == mParameter)
            {
                mParameter = null;
            }
            else
            {
                string ap = Convert.ToString(AttachPointNameOrIndexOrObject);
                if (ap == "AttachPoint1" || ap == "0")
                    mParameter = null;
            }
        }

        public void RemoveAll()
        {
            // Remove all parameters attached to the display. In this case, only 1 
            mParameter = null;           
        }

        public int get_AttachPointDepth(int index)
        {
            // In this function, we need to return how many parameters can be piled up on a single attachment point
            // In this case, we only have 1
            return 1;
        }

        #endregion

        #region ISynchronizeOutput Members

        // This function is called by the container when the user scrolls back or returns to streaming (realtime) mode
        // If realtime then the display is streaming data forward at the current time
        public bool IsRealTime
        {
			    set
			    {
				    mThisDisplayIsInRealTime = value;
			    }
        }

        public void PrepareOutput(IadsAutomationLib.IIadsTime parentContainerTime, IadsAutomationLib.IIadsTime globalTime)
        {
            // In this function, prepare all the graphics necessary for drawing and get ready for 
            // a "swap buffer" or "bit blit" to the screen.
            // Unless this is a high end 3D graphics control, then just do all the data processing and 
            // graphics work here
            if ( mParameter == null ) return;

            // This performs a "Goto Time" and gets the parameter ready to access data at the time 
            // specified by the parent container.
            mParameter.CurrentTime = parentContainerTime;

            // Step 1. Check to see if the user is in scroll back mode (and thus the time could be randomly jumping)
            //         or the display is streaming data at the current time
            if ( mThisDisplayIsInRealTime == false )
            {
                // The user is scrolling back with the scrollbar, so let's just read data from the current time and back 5 seconds
                // Or we could just simply read a single value at the current time and update the text value
                object dataValue = null;
                IadsAutomationLib.IIadsTime time = null;
                mParameter.Value2(IadsAutomationLib.IadsTimeProgressionType.iadsTimeNoChange, ref dataValue, ref time);
                label1.Text = Convert.ToString(dataValue);

                // Reset the state of the streaming frame read
                mParameter.HaveStartedFrameRead = false;

                return;
            }

            // Now here comes the fun part... We now have to focus on processing of "EveryDataPoint"
            // In this step, we need to keep track of the last time processed, and read all the 
            // data from this 'last time' to the current 'parentContainerTime'
            // There are several techniques to accomplish this. A simple while loop will suffice, but
            // you can also use the functions: StartFrameRead, GetNextFrameRead

            // Step 2. Check to see if we've started streaming data. If not, simply initialize the start time and return
            bool haveStartedStreaming = mParameter.HaveStartedFrameRead;
            if (!haveStartedStreaming)
            {
                // Initialize the frame read with the start time and maximum amount of data per read.
                // In this example, MaxSecondsPerFrame is set to 5 seconds. Keep this value resonable to protect against a time jump 
                // in realtime. For example, let's say time just randomly jumps an hour. In that case your display will attempt to process 1 hour 
                // of data and most likely lock up everything. Setting this value to something reasonable, prevents this situation.
                // In most cases, the time jump between successive calls to this display is much less than a tenth of a second... 
                // so 5 seconds will give more than enough leeway
                mParameter.StartFrameRead( parentContainerTime, 5.0 );
                return;
            }

            // Step 3. Read the frame of data from the last time to the current time. 
            object SafeArrayOfDataValues = null;
            object SafeArrayOfTimeValues = null;
            int numValuesReturned = 0;
            mParameter.GetNextFrameRead(parentContainerTime, ref SafeArrayOfDataValues, out numValuesReturned, ref SafeArrayOfTimeValues);

            // Step 4. Process all of the data. Of course we will be unable to display every value in the text object, so you'll need
            //         present the data using another mechanism such as a scrolled list or otherwise. If you're processing all the data
            //         to search for a specific event, simply search through all the data using a similar loop and update your graphical
            //         display accordingly
            Array dataArray = (Array)SafeArrayOfDataValues;
            for (int i = 0; i < numValuesReturned; i++)
            {
                long value = Convert.ToInt32(dataArray.GetValue(i));
                mTr.WriteLine( value );
            }

        }

        public void PerformOutput()
        {
            // Output Do final double buffer graphics swap here
            // Ignore for C# and do all the work in PrepareOutput
        }

        #endregion
    }

}
